from typing import List

from django.conf import settings
from django.contrib.auth.models import User
from django.db.models import Q, QuerySet
from django.test import override_settings

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.models import HelpDeskTicket
from django_spire.help_desk.notifications.handlers import TicketEventNotificationHandler
from django_spire.help_desk.tests.factories import create_helpdesk_ticket
from django_spire.notification.app.models import AppNotification
from django_spire.notification.email.models import EmailNotification

TEST_ADMINS = [
    ('developer1', 'developer1@stratus.com'),
    ('developer2', 'developer2@stratus.com'),
]


class TicketEventNotificationsHandlerTestCase(BaseTestCase):
    def _assert_user_notification_ticket_integrity(
            self,
            ticket: HelpDeskTicket,
            notification_type: type,
            users: QuerySet[User]):
        notifications = notification_type.objects.by_users(users)
        self.assertEqual(len(notifications), len(users))
        for notification in notifications:
            self.assertEqual(notification.notification.content_object, ticket)
            self.assertTrue(users.filter(pk=notification.notification.user.pk).exists())

    @override_settings(ADMINS=TEST_ADMINS)
    def test_handle_new(self):
        for admin in settings.ADMINS:
            User.objects.create_user(username=admin[0], email=admin[1], password='password')

        developers = User.objects.filter(Q(email__in=[admin[1] for admin in settings.ADMINS])).all()

        managers = User.objects.filter(
            Q(groups__permissions__codename='delete_helpdeskticket') |
            Q(is_superuser=True)).all()

        ticket = create_helpdesk_ticket()
        TicketEventNotificationHandler.handle_new(ticket=ticket)

        self._assert_user_notification_ticket_integrity(
            ticket=ticket,
            notification_type=AppNotification,
            users=managers)

        self._assert_user_notification_ticket_integrity(
            ticket=ticket,
            notification_type=AppNotification,
            users=developers)

        self._assert_user_notification_ticket_integrity(
            ticket=ticket,
            notification_type=EmailNotification,
            users=developers)




# Generated by Django 5.2 on 2025-05-17 23:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_ai_chat', '0003_rename__content_chatmessage__intel_data_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='chatmessage',
            name='response_type',
            field=models.CharField(choices=[('request', 'Request'), ('response', 'Response'), ('loading_response', 'Loading Response')], default='request', max_length=32),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='chatmessage',
            name='sender',
            field=models.CharField(default='', max_length=128),
            preserve_default=False,
        ),
    ]



from __future__ import annotations

from django.contrib.auth.models import Group
from django.urls import reverse

from django_spire.contrib.breadcrumb.breadcrumbs import Breadcrumbs
from django_spire.history.activity.mixins import ActivityMixin


class AuthGroup(Group, ActivityMixin):
    @classmethod
    def base_breadcrumb(cls) -> Breadcrumbs:
        crumbs = Breadcrumbs()
        crumbs.add_breadcrumb('Groups', reverse('permission:group_list'))
        return crumbs

    def breadcrumbs(self) -> Breadcrumbs:
        crumbs = Breadcrumbs()
        crumbs.add_base_breadcrumb(self._meta.model)

        if self.pk:
            crumbs.add_breadcrumb(
                name=self.name,
                href=reverse(
                    'permission:group_detail',
                    kwargs={'pk': self.pk}
                )
            )

        return crumbs

    class Meta:
        proxy = True
        verbose_name = 'Auth Group'
        verbose_name_plural = 'Auth Groups'





from typing import List

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.urls import reverse

from django_spire.help_desk.enums import TicketEventType
from django_spire.help_desk.exceptions import TicketEventNotificationTypeNotSupportedError
from django_spire.help_desk.models import HelpDeskTicket
from django_spire.notification.app.models import AppNotification
from django_spire.notification.choices import NotificationTypeChoices
from django_spire.notification.email.models import EmailNotification
from django_spire.notification.maps import NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP
from django_spire.notification.models import Notification


def get_ticket_event_notification_title(event_type: TicketEventType):
    content_map = {
        TicketEventType.NEW: "A new help desk ticket has been created",
    }

    return content_map[event_type]


def get_ticket_event_notification_body(event_type: TicketEventType, notification_type: NotificationTypeChoices, ticket: HelpDeskTicket):
    content_map = {
        TicketEventType.NEW: {
            NotificationTypeChoices.EMAIL: f"""
                                            <h2>A new help desk ticket (#{ticket.pk}) has been created.</h2>
                                            <p>Priority: {ticket.get_priority_display()}</p>
                                            <p>Purpose: {ticket.get_purpose_display()}</p>
                                            <p>Description:</p>
                                            <p>{ticket.description}</p>
                                        """,
            NotificationTypeChoices.APP: f'Priority: {ticket.get_priority_display()} - Purpose: {ticket.get_purpose_display()}'
        }
    }

    try:
        return content_map[event_type][notification_type]
    except KeyError:
        raise TicketEventNotificationTypeNotSupportedError(f'Combination of event type and notification type not supported: Event type {event_type} - Notification type {notification_type}')


def get_ticket_notification_url(ticket: HelpDeskTicket, notification_type: NotificationTypeChoices) -> str:
    path = reverse(
        "django_spire:help_desk:page:detail",
        kwargs={"pk": ticket.pk})

    if notification_type == NotificationTypeChoices.EMAIL:
        site_host = Site.objects.get_current()
        return f'{site_host}/{path[1:]}'

    return path


def create_ticket_event_notification_for_user(
        ticket: HelpDeskTicket,
        notification_type: NotificationTypeChoices,
        body: str,
        title: str,
        url: str,
        user: User,
    ):
    base_notification = Notification(
        content_type=ContentType.objects.get_for_model(ticket),
        object_id=ticket.pk,
        url=url,
        title=title,
        body=body,
        type=notification_type,
        user=user
    )

    if notification_type == NotificationTypeChoices.EMAIL:
        return EmailNotification(notification=base_notification,
                                 to_email_address=user.email)
    elif notification_type == NotificationTypeChoices.APP:
        return AppNotification(notification=base_notification)
    else:
        raise TicketEventNotificationTypeNotSupportedError(notification_type)


def create_ticket_event_notifications(
        ticket: HelpDeskTicket,
        users: List[User],
        notification_type: NotificationTypeChoices,
        event_type: TicketEventType):

    title = get_ticket_event_notification_title(event_type)
    body = get_ticket_event_notification_body(event_type, notification_type, ticket)
    url = get_ticket_notification_url(ticket, notification_type)

    notifications = [
        create_ticket_event_notification_for_user(
            ticket=ticket,
            notification_type=notification_type,
            body=body,
            title=title,
            url=url,
            user=user)
        for user in users
    ]

    Notification.objects.bulk_create([notification.notification for notification in notifications if notification])
    NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP[notification_type].objects.bulk_create(notifications)



from django.contrib.auth.models import User

from django_spire.help_desk.choices import HelpDeskTicketPriorityChoices, \
    HelpDeskTicketPurposeChoices, HelpDeskTicketStatusChoices
from django_spire.help_desk.models import HelpDeskTicket


def create_helpdesk_ticket_data(**kwargs) -> dict:
    ticket_data = {
        'priority': HelpDeskTicketPriorityChoices.LOW,
        'purpose': HelpDeskTicketPurposeChoices.APP,
        'status': HelpDeskTicketStatusChoices.READY,
        'description': 'This is a test ticket.',
    }

    ticket_data.update(**kwargs)

    return ticket_data

def create_helpdesk_ticket(**kwargs):
    if 'created_by' not in kwargs:
        kwargs['created_by'] = User.objects.first()

    return HelpDeskTicket.objects.create(**create_helpdesk_ticket_data(**kwargs))



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.contrib.auth.models import Permission, Group, User
from django.contrib.contenttypes.models import ContentType

from django_spire.auth.group.constants import (
    PERMISSION_MODELS_DICT,
    VALID_PERMISSION_LEVELS
)
from django_spire.auth.group.utils import (
    codename_list_to_perm_level,
    codename_to_perm_level,
    perm_level_to_string
)

if TYPE_CHECKING:
    from django.db.models import QuerySet
    from typing_extensions import Any


class ModelPermissions:
    def __init__(self, model_key: str):
        if model_key not in PERMISSION_MODELS_DICT:
            message = f'Model key {model_key} not in permission models dict.'
            raise KeyError(message)

        self.model = PERMISSION_MODELS_DICT[model_key]['model']
        self.is_proxy_model = PERMISSION_MODELS_DICT[model_key]['is_proxy_model']

        # All the permissions accessible to this model
        self.permissions: QuerySet[Permission] = self._set_model_perms()

    @property
    def app_label(self) -> str:
        # Used to generate codenames. For example delete_app_label
        return self.model._meta.app_label

    @property
    def model_name(self) -> str:
        return self.model._meta.model_name

    def find_permissions_by_level(
        self,
        perm_level: VALID_PERMISSION_LEVELS | None
    ) -> list[Permission]:
        permission_list = []

        for perm in self.permissions:
            if codename_to_perm_level(perm.codename) <= perm_level:
                permission_list.append(perm)

        return permission_list

    def get_special_role(self, codename: str) -> Permission | None:
        for perm in self.permissions:
            if perm.codename == codename:
                return perm

        return None

    def _set_model_perms(self) -> QuerySet[Permission]:
        content_type = ContentType.objects.get_for_model(
            self.model(),
            for_concrete_model=not self.is_proxy_model
        )

        return Permission.objects.filter(content_type=content_type)

    def special_role_list(self) -> list[Permission]:
        return [
            perm
            for perm in self.permissions
            if perm.codename.startswith('can')
        ]


class GroupPermissions:
    def __init__(self, group: Group, model_key: str):
        """
            Helper to use Django Groups as a cascading permission structure.
        """
        self.group = group
        self.model_permissions = ModelPermissions(model_key.lower())
        self.group_perms = self.group.permissions.all()

    def add_special_role(self, codename: str) -> None:
        special_role = self.model_permissions.get_special_role(codename)

        if special_role is not None:
            self.group.permissions.add(special_role)
            self.set_group_perms()

    def has_special_role(self, codename: str) -> bool:
        for perm in self.group_perms:
            if perm.codename == codename:
                return True

        return False

    def perm_level(self) -> VALID_PERMISSION_LEVELS:
        codename_list = []

        for perm in self.group_perms:
            if perm.codename.split('_')[-1] == self.model_permissions.model_name:
                codename_list.append(perm.codename)

        return codename_list_to_perm_level(codename_list)

    def perm_level_verbose(self) -> str:
        return perm_level_to_string(self.perm_level())

    @staticmethod
    def remove_special_permissions(perm_list: list | QuerySet) -> list:
        return [
            perm for perm in perm_list if not perm.codename.startswith('can')
        ]

    def update_perms(self, perm_level: VALID_PERMISSION_LEVELS) -> None:
        cascading_perms = self.remove_special_permissions(
            self.model_permissions.find_permissions_by_level(perm_level)
        )

        cascading_perms_queryset = Permission.objects.filter(
            pk__in=[perm.pk for perm in cascading_perms]
        )
        self.group.permissions.add(*cascading_perms_queryset)

        cascading_perms_id_list = [perm.pk for perm in cascading_perms]
        prohibited_permissions = self.remove_special_permissions(
            self.model_permissions.permissions.exclude(
                id__in=cascading_perms_id_list
            )
        )

        self.group.permissions.remove(*prohibited_permissions)

        self.set_group_perms()

    def remove_special_role(self, codename: str) -> None:
        special_role = self.model_permissions.get_special_role(codename)

        if special_role:
            self.group.permissions.remove(special_role)
            self.set_group_perms()

    def set_group_perms(self) -> None:
        self.group_perms = self.group.permissions.all()

    def toggle_special_role(self, codename: str) -> None:
        if self.has_special_role(codename):
            self.remove_special_role(codename)
        else:
            self.add_special_role(codename)


class UserPermissionHelper:
    def __init__(self, user: User, model_key: str):
        """
            Helper to query user's current cascading permissions
        """
        self.user = user
        self.model_permissions = ModelPermissions(model_key.lower())
        self.user_perms = self.get_user_perms()

    def get_user_perms(self) -> QuerySet[Permission]:
        return Permission.objects.filter(group__user=self.user).distinct()

    def perm_level(self) -> VALID_PERMISSION_LEVELS:
        codename_list = []

        for perm in self.user_perms:
            if perm.codename.split('_')[-1] == self.model_permissions.model_name:
                codename_list.append(perm.codename)

        return codename_list_to_perm_level(codename_list)

    def perm_level_verbose(self) -> str:
        return perm_level_to_string(self.perm_level())


def generate_user_perm_data(user: User) -> list[dict]:
    perm_data = []

    for key in PERMISSION_MODELS_DICT:
        user_permissions = UserPermissionHelper(user, key)

        perm_data.append({
            'app_name': key.capitalize(),
            'level_verbose': user_permissions.perm_level_verbose()
        })

    return perm_data


def generate_group_perm_data(
    group: Group,
    with_special_role: bool = False
) -> list[dict]:
    from django_spire.auth.group.constants import PERMISSION_MODELS_DICT

    perm_data = []

    for key in PERMISSION_MODELS_DICT:
        group_permissions = GroupPermissions(group=group, model_key=key)

        perm_information_dic = {
            'app_name': key.capitalize(),
            'level_verbose': group_permissions.perm_level_verbose()
        }

        if with_special_role:
            perm_information_dic['special_role_data'] = generate_special_role_data(group_permissions)

        perm_data.append(perm_information_dic)

    return perm_data


def generate_special_role_data(
    group_permissions: GroupPermissions
) -> list[dict[str, Any]]:
    special_role_data = []

    model_permissions = group_permissions.model_permissions

    for special_role in model_permissions.special_role_list():
        special_role_data.append({
            'name': special_role.name,
            'codename': special_role.codename,
            'has_access': group_permissions.has_special_role(special_role.codename)
        })

    return special_role_data



from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.tests.factories import create_helpdesk_ticket


class HelpDeskPageUrlsTestCase(BaseTestCase):
    def test_helpdesk_ticket_page_list_url_path(self) -> None:
        response = self.client.get(
            path=reverse('django_spire:help_desk:page:list'),
        )

        self.assertEqual(response.status_code, 200)

    def test_helpdesk_ticket_page_detail_url_path(self) -> None:
        response = self.client.get(
            path=reverse('django_spire:help_desk:page:detail', kwargs={'pk': create_helpdesk_ticket().pk}),
        )

        self.assertEqual(response.status_code, 200)


from __future__ import annotations

import functools

from typing_extensions import TYPE_CHECKING

from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.urls import reverse

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def permission_required(permissions: str | tuple[str, ...]) -> callable:
    def decorator(method: callable) -> callable:
        @functools.wraps(method)
        def wrapper(request: WSGIRequest, *args, **kwargs) -> callable | HttpResponseRedirect:
            if request.user.is_authenticated:
                if isinstance(permissions, str):
                    perms = (permissions,)
                else:
                    perms = permissions

                if not request.user.has_perms(perms):
                    raise PermissionDenied

                return method(request, *args, **kwargs)

            return HttpResponseRedirect(reverse('auth:admin:login'))

        return wrapper

    return decorator



from django.db import connections
from functools import wraps


def close_db_connections(func):
    @wraps(func)
    def inner(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        finally:
            connections.close_all()
    return inner



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from django.contrib.auth.models import User
    from django_spire.history.activity.mixins import ActivityMixin


def add_form_activity(model_object: ActivityMixin, pk: int, user: User) -> None:
    verb = (
        'created'
        if pk == 0 else 'updated'
    )

    information = (
        f'{user.get_full_name()} {verb} '
        f'{model_object._meta.verbose_name} "{model_object}".'
    )

    model_object.add_activity(
        user=user,
        verb=verb,
        information=information
    )





class FieldOverride:
    def __init__(self, seeder_class):
        self.seeder_class = seeder_class
        self.overrides = {}

    def __getattr__(self, name):
        """
            Delegate attribute lookup to self.seeder_class if the attribute isn't found
            in this FieldOverride instance.
        """

        attr = getattr(self.seeder_class, name)
        if callable(attr):
            def wrapper(*args, **kwargs):
                # Todo: Error here if fields is passed as an arg.
                # Pass the overrides field to the seed method
                if 'fields' in kwargs and isinstance(kwargs['fields'], dict):
                    kwargs['fields'] = {**kwargs['fields'], **self.overrides}
                else:
                    kwargs['fields'] = self.overrides

                return attr(*args, **kwargs)

            return wrapper
        return attr

    def filter(self, **kwargs):
        self.overrides.update(kwargs)
        return self

    def seed(self, count=1):
        return self.seeder_class.seed(count=count, fields=self.overrides)




from re import match
from typing import List

from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User
from django.urls import reverse



from django_spire.help_desk.exceptions import HelpDeskNotificationTypeNotSupportedError
from django_spire.help_desk.models import HelpDeskTicket
from django_spire.notification.choices import NotificationTypeChoices
from django_spire.notification.models import Notification
from django_spire.notification.app.models import AppNotification
from django_spire.notification.email.models import EmailNotification
from django_spire.notification.maps import NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP
from django_spire.notification.processors.notification import NotificationProcessor


class HelpDeskTicketNotifications:
    ticket: HelpDeskTicket

    def __init__(self, ticket: HelpDeskTicket):
        self.ticket = ticket

    def _create_helpdesk_notification_for_user(
            self,
            notification_type: NotificationTypeChoices,
            user: User,
            title: str =None,
            body: str =None,
            url: str =None
    ):
        base_notification = Notification(
            content_type=ContentType.objects.get_for_model(self.ticket),
            object_id=self.ticket.pk,
            url=url,
            title=title,
            body=body,
            type=notification_type,
            user=user
        )

        if notification_type == NotificationTypeChoices.EMAIL:
            return EmailNotification(notification=base_notification, to_email_address=user.email)

        elif notification_type == NotificationTypeChoices.APP:
            return AppNotification(notification=base_notification)

        else:
            raise HelpDeskNotificationTypeNotSupportedError(notification_type)

    def _create_helpdesk_notification_for_users(
            self,
            notification_type: NotificationTypeChoices,
            users: List[User],
            title: str =None,
            body: str =None,
            url: str =None
    ):
        notifications = [
            self._create_helpdesk_notification_for_user(notification_type, user, title, body, url)
            for user in users
        ]

        test = Notification.objects.bulk_create(list(map(lambda x: x.notification, notifications)))
        NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP[notification_type].objects.bulk_create(notifications)

    def send_ticket_created_notifications(self, title:str=None, body:str=None, url:str=None):
        if title is None:
            title = f'New Help Desk Ticket #{self.ticket.pk} has been created.'

        if body is None:
            body = (
                # f'A new help desk ticket (#{self.ticket.pk}) has been created.'
                # f'\n\nPurpose: {self.ticket.purpose}'
                # f'\n\nPriority: {self.ticket.priority}'
                # f'\n\nDescription: {self.ticket.description}'
                '<html><h1>New Help Desk Ticket</h1><p>A new help desk ticket has been created.</p></html>'
            )

        if url is None:
            url = reverse('django_spire:help_desk:page:detail', kwargs={'pk': self.ticket.pk})

        # TODO: Find out best way to get management user accounts to send app notifications to
        # management_users = get_management_users()
        management_users = [User.objects.get(username='stratus')]
        self._create_helpdesk_notification_for_users(NotificationTypeChoices.APP, management_users, title, body, url)

        # TODO: Find out best way to get developer user accounts to send email notifications to - iterate through them and send notifs to all
        # developer_users = get_developer_users()
        developer_users = management_users
        self._create_helpdesk_notification_for_users(NotificationTypeChoices.EMAIL, developer_users, title, body, url)

    def send_helpdesk_closed_notifications(self, title: str, body: str, url: str, to_email: str, publish_datetime=None):
        # TODO: Implement logic for sending closed notifications with different title and body
        raise NotImplementedError

    def send_helpdesk_comment_notifications(self, title: str, body: str, url: str, to_email: str, publish_datetime=None):
        # TODO: Implement logic for sending comment notifications with different title and body
        raise NotImplementedError


from __future__ import annotations

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Remove all migration(s) and delete SQLite database.'

    def handle(self, *_args, **_kwargs) -> None:
        is_migration_removed = self.remove_migration()

        if is_migration_removed:
            message = self.style.SUCCESS('The migration file(s) removed.')
            self.stdout.write(message)
        else:
            message = self.style.SUCCESS('The migration file(s) removed.')
            self.stdout.write(message)

        is_database_removed = self.remove_sqlite_database()

        if is_database_removed:
            message = self.style.SUCCESS('The test database was removed.')
            self.stdout.write(message)
        else:
            message = self.style.SUCCESS('The test database was removed.')
            self.stdout.write(message)

    def remove_migration(self) -> bool:
        found = False

        for app in settings.INSTALLED_APPS:
            path = settings.BASE_DIR / app.replace('.', '/') / 'migrations'

            if path.exists():
                for file in path.glob('*.py'):
                    if file.name == '__init__.py':
                        continue

                    file.unlink()
                    found = True

        return found

    def remove_sqlite_database(self) -> bool:
        path = settings.DATABASES.get('default')

        if not path:
            return False

        name = path.get('NAME')

        if name is None:
            return False

        path = settings.BASE_DIR / name

        if path.exists() and path.is_file():
            path.unlink()
            return True

        return False



from django_spire.notification.choices import NotificationStatusChoices
from django_spire.notification.sms.models import SmsTemporaryMedia


def update_unsent_notification_status_for_deleted_temporary_media(
    temporary_media_to_delete: list[SmsTemporaryMedia]
):
    for temporary_media in temporary_media_to_delete:
        if temporary_media.has_unsent_notifications():
            temporary_media.sms_notifications.all().update(
                notification__status=NotificationStatusChoices.ERRORED,
                notification__status_message='SMS temporary media expired before notification was sent',
            )



# Generated by Django 5.2.2 on 2025-06-11 21:04

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_notification_sms', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SmsTemporaryMedia',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField()),
                ('content_type', models.CharField(choices=[('image/png', 'Png'), ('image/jpeg', 'Jpeg')], max_length=64)),
                ('name', models.CharField(max_length=255)),
                ('expire_datetime', models.DateTimeField()),
                ('external_access_key', models.UUIDField(unique=True)),
            ],
            options={
                'verbose_name': 'SMS Temporary Media',
                'verbose_name_plural': 'SMS Temporary Media',
                'db_table': 'django_spire_notification_sms_temporary_media',
            },
        ),
        migrations.AddField(
            model_name='smsnotification',
            name='media_url',
            field=models.URLField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='smsnotification',
            name='temporary_media',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='sms_notifications', related_query_name='sms_notification', to='django_spire_notification_sms.smstemporarymedia'),
        ),
    ]



from __future__ import annotations

from typing_extensions import TYPE_CHECKING, Any

from django_spire.consts import __VERSION__
from django.urls import reverse

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest

def django_spire(_: WSGIRequest) -> dict[str, Any]:
    return {
        'DJANGO_SPIRE_VERSION': __VERSION__,
        'app_bootstrap_icon': {
            'help_desk': 'bi bi-headset'
        }
    }



from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.models import HelpDeskTicket
from django_spire.help_desk.tests.factories import create_helpdesk_ticket, create_helpdesk_ticket_data


class HelpDeskFormViewsTestCase(BaseTestCase):
    def test_helpdesk_ticket_form_create_view(self) -> None:
        ticket_data = create_helpdesk_ticket_data()

        response = self.client.post(
            data=ticket_data,
            path=reverse('django_spire:help_desk:form:create'),
        )

        ticket = HelpDeskTicket.objects.first()

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse('django_spire:help_desk:page:list'))

        self.assertEqual(ticket.priority, ticket_data['priority'])
        self.assertEqual(ticket.purpose, ticket_data['purpose'])
        self.assertEqual(ticket.status, ticket_data['status'])
        self.assertEqual(ticket.description, ticket_data['description'])


    def test_helpdesk_ticket_form_delete_view(self) -> None:
        ticket_pk = create_helpdesk_ticket().pk

        response = self.client.post(
            data={'should_delete': 'on'},
            path=reverse('django_spire:help_desk:form:delete', kwargs={'pk': ticket_pk}),
        )

        self.assertEqual(response.status_code, 302)
        self.assertTrue(HelpDeskTicket.objects.get(pk=ticket_pk).is_deleted)


    def test_helpdesk_ticket_form_update_view(self) -> None:
        ticket_pk = create_helpdesk_ticket().pk

        updated_ticket_data = create_helpdesk_ticket_data(
            priority='high',
            purpose='comp',
            status='prog',
            description='This is an updated test ticket.',
        )

        response = self.client.post(
            data=updated_ticket_data,
            path=reverse('django_spire:help_desk:form:update', kwargs={'pk': ticket_pk}),
        )

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse('django_spire:help_desk:page:list'))
        ticket = HelpDeskTicket.objects.get(pk=ticket_pk)
        self.assertEqual(ticket.priority, updated_ticket_data['priority'])
        self.assertEqual(ticket.purpose, updated_ticket_data['purpose'])
        self.assertEqual(ticket.status, updated_ticket_data['status'])
        self.assertEqual(ticket.description, updated_ticket_data['description'])


from __future__ import annotations

from django.db.models import QuerySet


class FileQuerySet(QuerySet):
    def active(self):
        return self.filter(is_deleted=False, is_active=True)

    def related_field(self, field_name: str):
        return self.filter(related_field=field_name)



import json

from django.conf import settings
from django.http import HttpResponse

from django_spire.ai.chat.messages import DefaultMessageIntel
from django_spire.ai.chat.models import Chat
from django_spire.ai.chat.responses import MessageResponse
from django_spire.ai.chat.choices import MessageResponseType
from django_spire.ai.chat.tools import chat_workflow_process
from django_spire.consts import AI_CHAT_WORKFLOW_SENDER_SETTINGS_NAME


def response_message_render_view(request):
    body_data = json.loads(request.body)

    chat_workflow_name = getattr(settings, AI_CHAT_WORKFLOW_SENDER_SETTINGS_NAME)

    if chat_workflow_name is None:
        raise ValueError(
            f'"{AI_CHAT_WORKFLOW_SENDER_SETTINGS_NAME}" must be set in the django settings.'
        )

    chat = Chat.objects.by_user(request.user).get(id=body_data['chat_id'])

    message_intel = chat_workflow_process(
        request,
        body_data['message_body'],
        message_history=chat.generate_message_history(),
    )

    response_message = MessageResponse(
        type=MessageResponseType.RESPONSE,
        sender=chat_workflow_name,
        message_intel=message_intel,
        synthesis_speech=body_data.get('synthesis_speech', False),
    )

    chat.add_message_response(response_message)

    return HttpResponse(response_message.render_to_html_string({'chat_id': chat.id}))



from django_spire.contrib.service.service import BaseService
from django_spire.contrib.service.django_model_service import BaseDjangoModelService

__all__ = [
    'BaseService',
    'BaseDjangoModelService'
]



from __future__ import annotations

import json
from datetime import datetime, timedelta
from typing import Any

from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpRequest


class SessionController:
    """
        This class provides an interface for storing, retrieving, and managing session data
        with automatic expiration. It handles session data under a specific key and supports
        timeout-based cleanup of expired sessions.

        Args:
            request (HttpRequest): The Django request object containing the session.
            session_key (str): Unique key under which the session data will be stored.
            seconds_till_expiry (int, optional): Number of seconds until session expiry.
                Defaults to 300 seconds (5 minutes).
    """

    _TIMEOUT_KEY = '_timeout_datestamp'

    def __init__(
            self,
            request: HttpRequest,
            session_key: str,
            seconds_till_expiry: int = 60 * 5
    ):
        self.request = request
        self.session_key = session_key
        self.seconds_till_expiry = seconds_till_expiry

        self.request.session.setdefault(self.session_key, dict())
        self._session = self.request.session[self.session_key]

        self._clean()

    def __getitem__(self, key: str) -> Any:
        return self.data[key]

    def __setitem__(self, key: str, value: Any):
        self.add_data(key, value)

    def add_data(self, key: str, data: Any):
        self._session[key] = data
        self._set_timeout_datestamp()
        self._set_modified()

    @property
    def data(self):
        return self._session

    @property
    def is_expired(self) -> bool:
        current_timestamp = datetime.now().timestamp()
        return self.timeout_datestamp < current_timestamp

    @property
    def timeout_datestamp(self) -> float:
        return self.data.get(self._TIMEOUT_KEY, 0)

    def remove_data(self, key: str):
        self.data.pop(key)
        self._set_modified()

        # remove the session completely if there is no data in it.
        if self._TIMEOUT_KEY in self.data and len(self.data.keys()) == 1:
            self.data.pop(self._TIMEOUT_KEY)

    def purge(self):
        self.request.session.pop(self.session_key)
        self._set_modified()

    def _clean(self) -> None:

        if self._TIMEOUT_KEY in self.data and self.is_expired:
            self.request.session.pop(self.session_key)
            self._set_modified()

    def _set_modified(self):
        self.request.session.modified = True

    @property
    def has_data(self) -> bool:
        return bool(self.data)

    def _set_timeout_datestamp(self):
        timeout_datetime = datetime.now() + timedelta(seconds=self.seconds_till_expiry)
        self.data[self._TIMEOUT_KEY] = timeout_datetime.timestamp()

    def to_json(self):
        return json.dumps(self.data, cls=DjangoJSONEncoder)



from __future__ import annotations

from django import forms
from django_spire.file.fields import MultipleFileField


class FileForm(forms.Form):
    files = MultipleFileField()

    # def __init__(self, *args, **kwargs):
    #     super().__init__(*args, **kwargs)
    #
    #     self.helper = FormHelper(self)
    #     self.helper.include_media = False
    #     self.helper.form_enctype = 'multipart/form-data'
    #     self.helper.layout = Layout(
    #         Row(
    #             Column('files', css_class='form-group col-md-6'),
    #         ),
    #     )
    #     self.helper.add_input(Submit('submit', 'Submit', css_class='btn-primary'))




from typing import ClassVar

from django import forms
from django.contrib.auth.models import User

from django_spire.help_desk.models import HelpDeskTicket


class HelpDeskTicketCreateForm(forms.ModelForm):
    def save(self, commit: bool = True, user: User | None = None) -> HelpDeskTicket:
        if user is not None:
            self.instance.created_by = user

        return super().save(commit=commit)

    class Meta:
        model = HelpDeskTicket
        exclude: ClassVar[list] = ['created_by', 'status']


class HelpDeskTicketUpdateForm(forms.ModelForm):
    class Meta:
        model = HelpDeskTicket
        exclude: ClassVar[list] = ['created_by']


from django.db.models import Q, Count

from django_spire.history.querysets import HistoryQuerySet


class ChatQuerySet(HistoryQuerySet):
    def by_user(self, user):
        return self.filter(
            user=user,
        )

    def get_empty_or_create(self, user):
        try:
            return (
                self.filter(user=user)
                .annotate(num_messages=Count('message'))
                .filter(num_messages=0)
                .earliest('-id')
            )
        except self.model.DoesNotExist:
            return self.create(user=user)

    def search(self, query: str):
        return self.filter(
            Q(name__icontains=query)
        )


class ChatMessageQuerySet(HistoryQuerySet):
    def newest_by_count(self, count: int = 20):
        return self.order_by('-created_datetime')[:count]

    def newest_by_count_reversed(self, count: int = 20):
        return self.order_by('-created_datetime')[:count][::-1]


import unittest
from django_spire.contrib.seeding.field.callable import CallableFieldSeeder


class TestCallableFieldSeeder(unittest.TestCase):

    def test_seeds_callable_fields(self):
        fields = {
            "timestamp": lambda: "2024-01-01 12:00:00",
            "status": lambda: "active"
        }
        seeder = CallableFieldSeeder(fields=fields)
        result = seeder.seed(None, count=3)

        self.assertEqual(len(result), 3)
        for row in result:
            self.assertEqual(row["timestamp"], "2024-01-01 12:00:00")
            self.assertEqual(row["status"], "active")



from django.urls import include, path

app_name = 'chat'

urlpatterns = [
    path('json/', include('django_spire.ai.chat.urls.json_urls', namespace='json')),
    path('message/', include('django_spire.ai.chat.urls.message_urls', namespace='message')),
    path('template/', include('django_spire.ai.chat.urls.template_urls', namespace='template')),
]



from django.conf import settings
from django.contrib.auth.models import User
from django.db.models import Q

from django_spire.help_desk.notifications.factories import create_ticket_event_notifications
from django_spire.help_desk.models import HelpDeskTicket
from django_spire.help_desk.enums import TicketEventType
from django_spire.notification.choices import NotificationTypeChoices


class TicketEventNotificationHandler:
    @staticmethod
    def handle_new(ticket: HelpDeskTicket):
        event_type = TicketEventType.NEW

        developers = User.objects.filter(
            email__in=[
                admin[1]
                for admin in settings.ADMINS
            ]).all()

        for notif_type in [NotificationTypeChoices.APP, NotificationTypeChoices.EMAIL]:
            create_ticket_event_notifications(
                ticket=ticket,
                users=developers,
                notification_type=notif_type,
                event_type=event_type,
            )

        managers = User.objects.filter(
            Q(groups__permissions__codename='delete_helpdeskticket') |
            Q(is_superuser=True)).all()

        create_ticket_event_notifications(
            ticket=ticket,
            users=managers,
            notification_type=NotificationTypeChoices.APP,
            event_type=event_type,
        )


from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.tests.factories import create_helpdesk_ticket


class HelpDeskPageViewsTestCase(BaseTestCase):
    def test_helpdesk_ticket_page_list_view(self) -> None:
        tickets = [create_helpdesk_ticket() for _ in range(5)]

        response = self.client.get(
            path=reverse('django_spire:help_desk:page:list'),
        )

        self.assertEqual(response.status_code, 200)
        for ticket in tickets:
            self.assertIn(ticket, response.context_data['tickets'])

    def test_helpdesk_ticket_page_detail_view(self) -> None:
        ticket = create_helpdesk_ticket()

        response = self.client.get(
            path=reverse('django_spire:help_desk:page:detail', kwargs={'pk': ticket.pk}),
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context_data['ticket'], ticket)


from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from django.utils.http import urlencode

from django_spire.notification.sms.models import SmsNotification, SmsTemporaryMedia


@admin.register(SmsNotification)
class SmsNotificationAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'view_notification_link', 'to_phone_number', 'media_url', 'temporary_media'
    )
    list_filter = (
        ('temporary_media', admin.EmptyFieldListFilter),
        ('media_url', admin.EmptyFieldListFilter),
    )
    list_select_related = ('notification', )

    def view_notification_link(self, sms_notification: SmsNotification):
        url = (
                reverse("admin:django_spire_notification_notification_changelist")
                + "?"
                + urlencode({"id": f"{sms_notification.notification_id}"})
        )

        return format_html(f'<a href="{url}">{sms_notification.notification}</a>')

    view_notification_link.short_description = "Notification"


@admin.register(SmsTemporaryMedia)
class SmsTemporaryMediaAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'name', 'content_type', 'external_url'
    )



import json

from django.template.response import TemplateResponse
from django.utils.timezone import now

from django_spire.ai.chat.models import Chat
from django_spire.core.shortcuts import get_object_or_null_obj


def confirm_delete_view(request, pk: int) -> TemplateResponse:
    return TemplateResponse(
        request,
        context={'chat': get_object_or_null_obj(Chat, pk=pk)},
        template='django_spire/ai/chat/section/confirm_delete_section.html'
    )


def dialog_widget_view(request) -> TemplateResponse:
    body_data = json.loads(request.body)

    chat_id = body_data['chat_id']

    if chat_id == 0:
        chat = Chat.objects.get_empty_or_create(
            user=request.user
        )
        chat.name = 'New Chat'
        chat.last_message_datetime = now()
        chat.save()

    else:
        chat = Chat.objects.get(
            id=chat_id,
            user=request.user,
        )

    return TemplateResponse(
        request,
        context={
            'chat': chat,
            'top_nav_height_px': body_data['top_nav_height_px']
        },
        template='django_spire/ai/chat/widget/dialog_widget.html'
    )


def recent_chats_widget_view(request) -> TemplateResponse:
    return TemplateResponse(
        request,
        context={
            'recent_chats': (
                Chat.objects
                .by_user(request.user)
                .order_by('-last_message_datetime')
                .active()
            )[:20]
        },
        template='django_spire/ai/chat/widget/recent_chat_list_widget.html'
    )


def search_chats_results_widget_view(request) -> TemplateResponse:
    body_data = json.loads(request.body)
    chats = Chat.objects.by_user(request.user).search(body_data['query'])

    return TemplateResponse(
        request,
        context={
            'search_results': [
                {
                    'name': chat.name,
                    'id': chat.id,
                } for chat in chats
            ]
        },
        template='django_spire/ai/chat/widget/search_chat_results_widget.html'
    )



from __future__ import annotations

import logging
from abc import ABC
from typing import Type, Generic, TypeVar

from django.db import transaction
from django.db.models import Model

from django_spire.contrib.service.exceptions import ServiceException
from django_spire.contrib.service.service import BaseService

TypeDjangoModel = TypeVar('TypeDjangoModel', bound=Model, covariant=True)


class BaseDjangoModelService(BaseService[TypeDjangoModel], ABC, Generic[TypeDjangoModel]):
    def _get_concrete_fields(self) -> dict:
        return {
            field.name: field
            for field in self.obj._meta.get_fields()
            if field.concrete and not field.many_to_many and not field.one_to_many
        }

    def _get_touched_fields(self, concrete_fields: dict, **field_data: dict) -> list[str]:
        foreign_key_id_aliases = {f"{name}_id" for name, field in concrete_fields.items() if field.many_to_one}
        allowed = set(concrete_fields) | foreign_key_id_aliases

        touched_fields: list[str] = []

        for field, value in field_data.items():
            if field not in allowed:
                logging.warning(f'Field {field!r} is not valid for {self.obj.__class__.__name__}')
                continue

            model_field = concrete_fields.get(field.rstrip("_id"), None)

            if model_field and (getattr(model_field, 'auto_created', False) or not model_field.editable):
                continue

            setattr(self.obj, field, value)

            touched_fields.append(field.rstrip('_id'))

        return touched_fields

    @property
    def _model_obj_id_is_empty(self) -> bool:
        return self.obj.id is None or self.obj.id == 0 or self.obj.id == ''

    @property
    def _model_obj_pk_is_empty(self) -> bool:
        return self.obj.pk is None or self.obj.pk == 0 or self.obj.pk == ''

    @property
    def model_obj_is_created(self) -> bool:
        return self._obj_is_valid and not self.model_obj_is_new

    @property
    def model_obj_is_new(self) -> bool:
        return self._model_obj_id_is_empty or self._model_obj_pk_is_empty

    def validate_model_obj(self, **field_data: dict) -> list[str]:
        concrete_fields = self._get_concrete_fields()
        touched_fields = self._get_touched_fields(concrete_fields, **field_data)

        try:
            self.obj.full_clean(
                exclude=[field for field in concrete_fields if field not in touched_fields]
            )
        except:
            raise

        return touched_fields

    @transaction.atomic
    def save_model_obj(self, **field_data: dict) -> bool:
        new_model_obj_was_created = False

        if not field_data:
            raise ServiceException(f'Field data is required to save on {self.obj.__class__.__name__}')

        touched_fields = self.validate_model_obj(**field_data)

        if self.model_obj_is_new:
            new_model_obj_was_created = True
            self.obj.save()

        elif touched_fields:
            self.obj.save(update_fields=touched_fields)

        else:
            logging.warning(
                f'{self.obj.__class__.__name__} is not a new object or there was no touched fields to update.')

        return new_model_obj_was_created

    @property
    def obj_class(self) -> Type[TypeDjangoModel]:
        return super().obj_class

    @property
    def _obj_is_valid(self) -> bool:
        return super()._obj_is_valid and isinstance(self.obj, Model) and issubclass(self._obj_type, Model)



import django_glue as dg
from django.shortcuts import redirect, get_object_or_404
from django.urls import reverse

from django_spire.auth.group.decorators import permission_required
from django_spire.contrib.form.confirmation_forms import DeleteConfirmationForm
from django_spire.contrib.form.utils import show_form_errors
from django_spire.contrib.generic_views import portal_views
from django_spire.help_desk import forms
from django_spire.help_desk.notifications.handlers import TicketEventNotificationHandler
from django_spire.help_desk.models import HelpDeskTicket


@permission_required('django_spire_help_desk.delete_helpdeskticket')
def ticket_delete_form_view(request, pk: int = 0):
    ticket = get_object_or_404(HelpDeskTicket, pk=pk)

    if request.method == 'POST':
        form = DeleteConfirmationForm(request.POST, obj=ticket)

        if form.is_valid():
            ticket.set_deleted()

            return redirect(reverse('django_spire:help_desk:page:list'))

        show_form_errors(request, form)

    else:
        form = DeleteConfirmationForm(request.GET, obj=ticket)

    return portal_views.form_view(
        request,
        form=form,
        verb=f'Delete',
        obj=ticket,
    )


@permission_required('django_spire_help_desk.change_helpdeskticket')
def ticket_create_form_view(request):
    ticket = HelpDeskTicket()

    dg.glue_model_object(request, 'ticket', ticket)

    if request.method == 'POST':
        form = forms.HelpDeskTicketCreateForm(request.POST)

        if form.is_valid():
            ticket = form.save(user=request.user)
            TicketEventNotificationHandler.handle_new(ticket)

            return redirect(reverse('django_spire:help_desk:page:list'))

        show_form_errors(request, form)

    else:
        form = forms.HelpDeskTicketCreateForm(instance=ticket)

    return portal_views.form_view(
        request,
        form=form,
        template='django_spire/help_desk/page/ticket_form_page.html',
        verb=f'Create',
        obj=ticket,
        context_data={
            'form_action_url': reverse('django_spire:help_desk:form:create'),
        }
    )


@permission_required('django_spire_help_desk.change_helpdeskticket')
def ticket_update_form_view(request, pk: int):
    ticket = get_object_or_404(HelpDeskTicket, pk=pk)

    dg.glue_model_object(request, 'ticket', ticket)

    if request.method == 'POST':
        form = forms.HelpDeskTicketUpdateForm(request.POST, instance=ticket)

        if form.is_valid():
            ticket = form.save()

            return redirect(reverse('django_spire:help_desk:page:list'))

        show_form_errors(request, form)

    else:
        form = forms.HelpDeskTicketUpdateForm(instance=ticket)

    return portal_views.model_form_view(
        request=request,
        obj=ticket,
        template='django_spire/help_desk/page/ticket_form_page.html',
        form=form,
        context_data={
            'ticket': ticket,
            'form_action_url': reverse('django_spire:help_desk:form:update', kwargs={'pk': ticket.pk}),
        }
    )


from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.tests.factories import create_helpdesk_ticket


class HelpDeskFormUrlsTestCase(BaseTestCase):
    def test_helpdesk_ticket_form_create_url_path(self) -> None:
        response = self.client.get(
            path=reverse('django_spire:help_desk:form:create'),
        )

        self.assertEqual(response.status_code, 200)

    def test_helpdesk_ticket_form_delete_url_path(self) -> None:
        response = self.client.get(
            path=reverse('django_spire:help_desk:form:delete', kwargs={'pk': create_helpdesk_ticket().pk}),
        )

        self.assertEqual(response.status_code, 200)

    def test_helpdesk_ticket_form_update_url_path(self) -> None:
        response = self.client.get(
            path=reverse('django_spire:help_desk:form:update', kwargs={'pk': create_helpdesk_ticket().pk}),
        )

        self.assertEqual(response.status_code, 200)





from abc import ABC, abstractmethod

from django_spire.contrib.seeding.field.cleaners import normalize_seeder_fields
from django_spire.contrib.seeding.field.enums import FieldSeederTypesEnum


class BaseFieldSeeder(ABC):
    keyword: str = None
    seed_keywords = FieldSeederTypesEnum._value2member_map_.keys()

    def __init__(
            self,
            fields: dict = None,
            default_to: str = "llm"
    ):

        self.fields = self._normalize_fields(fields or {})
        self.default_to = default_to


    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)

        if cls.keyword is None:
            raise ValueError("Seeds must have a keyword")

    def _normalize_fields(self, fields: dict) -> dict:
        return normalize_seeder_fields(fields)

    def filter_fields(self, seed_type: str) -> dict:
        return {
            k: v for k, v in self.fields.items()
            if isinstance(v, tuple) and v[0] == seed_type
        }

    @property
    def seeder_fields(self):
        return self.filter_fields(self.keyword)

    @abstractmethod
    def seed(self, model_seeder_cls, count: int):
        pass



# Generated by Django 5.2.1 on 2025-05-23 16:00

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='HelpDeskTicket',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('purpose', models.CharField(choices=[('app', 'App'), ('comp', 'Company')], max_length=4, verbose_name='Purpose')),
                ('priority', models.CharField(choices=[('low', 'Low'), ('med', 'Medium'), ('high', 'High'), ('urge', 'Urgent')], max_length=4, verbose_name='Priority')),
                ('status', models.CharField(choices=[('read', 'Ready'), ('prog', 'In Progress'), ('done', 'Done')], default='read', max_length=4, verbose_name='Status')),
                ('description', models.TextField()),
                ('created_by', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, related_name='created_help_desk_tickets', related_query_name='created_help_desk_ticket', to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
            ],
            options={
                'verbose_name': 'Help Desk Ticket',
                'verbose_name_plural': 'Help Desk Tickets',
                'db_table': 'django_spire_help_desk_ticket',
            },
        ),
    ]



from django.urls import include, path
from django.views.generic import TemplateView

app_name = 'core'

urlpatterns = [
    path('403/', TemplateView.as_view(template_name='django_spire/403.html'), name='403'),
    path('404/', TemplateView.as_view(template_name='django_spire/404.html'), name='404'),
    path('500/', TemplateView.as_view(template_name='django_spire/500.html'), name='500'),
    path('maintenance/', TemplateView.as_view(template_name='django_spire/maintenance.html'), name='maintenance')
]



from __future__ import annotations

from django import forms
from django.contrib.auth.models import User

from django_spire.auth.user.models import AuthUser
from django_spire.auth.user.factories import register_new_user


class UserForm(forms.ModelForm):
    def save(self, commit: bool = True):
        self.instance.username = self.cleaned_data['email']
        return super().save(commit=commit)

    class Meta:
        model = AuthUser
        fields = ['first_name', 'last_name', 'email', 'is_active']


class AddUserForm(forms.ModelForm):
    # def __init__(self, *args, **kwargs):
    #     super().__init__(*args, **kwargs)
    #
    #     self.helper = FormHelper(self)
    #     self.helper.include_media = False
    #
    #     self.fields['first_name'].required = True
    #     self.fields['last_name'].required = True
    #     self.fields['email'].required = True
    #
    #     self.helper.layout = Layout(
    #         Row(
    #             Column('first_name', css_class='form-group col-md-6 col-12'),
    #             Column('last_name', css_class='form-group col-md-6 col-12')
    #         ),
    #         Row(
    #             Column(Field('email', autocomplete='off'), css_class='form-group col-md-6 col-12'),
    #             Column(Field('password', autocomplete='off'), css_class='form-group col-md-6 col-12'),
    #         ),
    #     )
    #     self.helper.add_input(Submit('submit', 'Submit', css_class='btn-primary btn-sm bg-blue'))

    class Meta:
        model = User
        exclude = ['date_joined', 'last_login', 'id', 'username']
        widgets = {
            'password': forms.PasswordInput()
        }


class UserGroupForm(forms.Form):
    def __init__(self, *args, **kwargs):
        from django.contrib.auth.models import Group
        self.user = kwargs.pop('user')
        super().__init__(*args, **kwargs)

        self.helper = FormHelper(self)
        self.helper.include_media = False

        user_groups = self.user.groups.all()

        self.fields['group_list'] = forms.ModelMultipleChoiceField(
            queryset=Group.objects.exclude(pk__in=[group.pk for group in user_groups]),
            initial=user_groups,
            required=True,
            label='Available Groups'
        )

        self.helper.layout = Layout(
            Row(
                Column('group_list', css_class='form-group col-12'),
            ),
        )
        self.helper.add_input(Submit('submit', 'Submit', css_class='btn-primary btn-sm bg-blue'))


class EditUserForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['first_name'].required = True
        self.fields['last_name'].required = True
        self.fields['email'].required = True

    class Meta:
        model = User
        fields = ['first_name', 'last_name', 'email']


class RegisterUserForm(forms.ModelForm):
    def clean_password(self) -> str:
        password = self.cleaned_data.get('password')

        if len(password) < 8:
            message = 'Must be at least 8 characters long.'
            raise forms.ValidationError(message)

        return password

    def save(self, commit: bool = False):
        return register_new_user(**self.cleaned_data)

    class Meta:
        model = AuthUser
        fields = ['email', 'password', 'first_name', 'last_name']



from django.apps import AppConfig

from django_spire.utils import check_required_apps


class UserAccountConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    label = 'django_spire_auth_user'
    name = 'django_spire.auth.user'

    REQUIRED_APPS = ('django_spire_core', 'django_spire_auth')

    def ready(self) -> None:
        check_required_apps(self.label)
